@@ -1,4 +1,4 @@
-# Copyright 2004-2012 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2013 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -58,7 +58,7 @@ def register_prefix(prefix, prio, addprefixes=[]):
         alts2 = prefixes
 
     alts2 += addprefixes
-        
+
     alts = [ a1 + a2 for a1 in alts1 for a2 in alts2 ]
 
     prefix_subs[prefix] = prio, alts
@@ -108,7 +108,21 @@ def expand_anchor(v):
     Turns an anchor into a number.
     """
 
-    return anchors.get(v, v)
+    try:
+        return anchors.get(v, v)
+    except:
+        # This fixes some bugs in very old Ren'Pys.
+
+        for n in anchors:
+            o = getattr(renpy.store, n, None)
+            if o is None:
+                continue
+
+            if v is o:
+                return anchors[n]
+
+        raise
+
 
 
 # A map of properties that we know about. The properties may take a
@@ -118,6 +132,7 @@ style_properties = dict(
     aft_bar = none_is_null,
     aft_gutter = None,
     antialias = None,
+    vertical = None,
     background = renpy.easy.displayable_or_none,
     bar_invert = None,
     bar_resizing = None,
@@ -128,6 +143,7 @@ style_properties = dict(
     bottom_margin = None,
     bottom_padding = None,
     box_layout = None,
+    box_reverse = None,
     box_wrap = None,
     caret = renpy.easy.displayable_or_none,
     child = renpy.easy.displayable_or_none,
@@ -157,6 +173,7 @@ style_properties = dict(
     mouse = None,
     min_width = None,
     newline_indent = None,
+    order_reverse = None,
     outlines = expand_outlines,
     rest_indent = None,
     right_margin = None,
@@ -191,8 +208,8 @@ style_properties = dict(
     ymaximum = None,
     yminimum = None,
     yoffset = None,
-    ypos = None, 
-    hyperlink_functions=None,   
+    ypos = None,
+    hyperlink_functions=None,
     line_overlap_split=None,
     )
 
@@ -237,18 +254,18 @@ substitutes = dict(
         ('left_padding', None),
         ('right_padding', None),
         ],
-    
+
     ypadding = [
         ('top_padding', None),
         ('bottom_padding', None),
         ],
-    
+
     minwidth = [ ('min_width', None) ],
     textalign = [ ('text_align', None) ],
     slow_speed = [ ('slow_cps', None) ],
     enable_hover = [ ],
     left_gutter = [ ('fore_gutter', None) ],
-    right_gutter = [ ('aft_gutter', None) ], 
+    right_gutter = [ ('aft_gutter', None) ],
     top_gutter = [ ('fore_gutter', None) ],
     bottom_gutter = [ ('aft_gutter', None) ],
     left_bar = [ ('fore_bar', none_is_null) ],
@@ -262,18 +279,18 @@ substitutes = dict(
         ('xpos', index_0),
         ('ypos', index_1),
         ],
-    
+
     anchor = [
         ('xanchor', index_0),
         ('yanchor', index_1),
         ],
 
-    # Conflicts w/ a variable used in the Style implementation.    
+    # Conflicts w/ a variable used in the Style implementation.
     # offset = [
     #     ('xoffset', index_0),
     #     ('yoffset', index_1),
     #     ],
-    
+
     align = [
         ('xpos', index_0),
         ('ypos', index_1),
@@ -290,7 +307,7 @@ substitutes = dict(
         ('xminimum', index_0),
         ('yminimum', index_1),
         ],
-    
+
     area = [
         ('xpos', index_0),
         ('ypos', index_1),
@@ -313,7 +330,7 @@ substitutes = dict(
         ('ypos', None),
         ('yanchor', always_half),
         ],
-    
+
     )
 
 # Map from property to number.
@@ -333,14 +350,14 @@ property_numbers = 0
 def init():
 
     global property_numbers
-    
+
     # Figure out a map from style property name to an (arbitrary,
     # session-specific) style property number.
     for i, p in enumerate(style_properties):
         property_number[p] = i
 
     # Figure out a map from style prefix to style property number offset.
-    property_numbers = 0    
+    property_numbers = 0
     for r in roles:
         for p in prefixes:
             prefix_offset[r + p] = property_numbers
@@ -363,15 +380,15 @@ def init():
 
             for real_prop, function in replacements:
                 propn = property_number[real_prop]
-                
+
                 for a in alts:
                     expansions[prefix + virtual_prop].append((prio, propn + prefix_offset[a], function))
 
     # Cache mappings for position properties.
     for i in ('xpos', 'xanchor', 'xoffset', 'ypos', 'yanchor', 'yoffset', 'subpixel'):
         globals()["prop_" + i] = property_number[i]
-                
-            
+
+
 init()
 
 # A map from a style name to the style associated with that name.
@@ -401,7 +418,7 @@ def reset():
     global styles_built
     global styles_pending
     global style_help
-    
+
     style_map = { }
     style_help = { }
     style_parts = collections.defaultdict(dict)
@@ -416,7 +433,7 @@ class StyleManager(object):
 
     def __getattr__(self, name):
         global styles_built
-        
+
         try:
             return style_map[name]
         except:
@@ -426,26 +443,26 @@ class StyleManager(object):
         if "_" in name:
 
             rest = name
-            
+
             while "_" in rest:
                 _first, rest = rest.split("_", 1)
 
                 if rest in style_map:
-                    
+
                     s = Style(rest)
-                    self.__setattr__(name, s, False)                    
-                    
+                    self.__setattr__(name, s, False)
+
                     return s
 
             raise Exception("The style %s does not exist, and couldn't be auto-created because %s doesn't exist, either." % (name, rest))
-                
+
         raise Exception('The style %s does not exist.' % name)
 
     def __setattr__(self, name, value, check_built=True):
 
         if check_built and styles_built:
             raise Exception("Cannot assign to style outside of the init phase.")
-        
+
         if isinstance(value, Style):
             if value.name is None:
                 value.name = (name, )
@@ -454,7 +471,7 @@ class StyleManager(object):
             style_parts[name][()] = value
         else:
             object.__setattr__(self, name, value)
-        
+
     def create(self, name, parent, description=None):
         """
         Creates a new style.
@@ -473,12 +490,12 @@ class StyleManager(object):
 
     def rebuild(self):
         renpy.style.rebuild()
-        
+
     def exists(self, name):
         """
         This determines if the named style exists.
         """
-        
+
         return name in style_map
 
     def get(self, name):
@@ -487,12 +504,12 @@ class StyleManager(object):
             name = (name, )
 
         s = style_map
-            
+
         for i in name:
             s = s[i]
 
         return s
-    
+
 def expand_properties(properties):
 
     rv = [ ]
@@ -500,7 +517,7 @@ def expand_properties(properties):
     for prop, val in properties.iteritems():
 
         oldfunc = None
-        
+
         try:
             e = expansions[prop]
         except KeyError:
@@ -514,7 +531,7 @@ def expand_properties(properties):
                     newval = func(val)
             else:
                 newval = val
-                                
+
             rv.append((prio, propn, newval))
 
     # Places things in priority order... so more important properties
@@ -523,16 +540,16 @@ def expand_properties(properties):
     return rv
 
 
-# This builds the style. 
+# This builds the style.
 def build_style(style):
 
     if style.cache is not None:
         return
 
     updates = [ ]
-    
+
     if style.parent is not None:
-        
+
         name = style.parent
 
         # The left base is the style that shares the most indexes with
@@ -545,12 +562,12 @@ def build_style(style):
 
             # The down bases inherit from the parents of the unindexd
             # style, but don't have as many components.
-            
+
             if rest:
                 down_base = [ style_map[first] ]
             else:
                 down_base = [ ]
-            
+
             while first:
 
                 left_base = style_parts[first].get(rest, None)
@@ -562,7 +579,7 @@ def build_style(style):
                     ss = style_map[first]
                 except KeyError:
                     ss = getattr(renpy.game.style, first)
-                
+
                 down_base.insert(0, ss)
                 first = ss.parent and ss.parent[0]
 
@@ -576,14 +593,14 @@ def build_style(style):
 
                 if ss.cache is None:
                     build_style(ss)
-                
+
                 updates.extend(ss.updates)
 
             for j in rest:
 
                 if ss.indexed is None:
                     break
-                
+
                 ss = ss.indexed.get(j, None)
 
                 if ss is None:
@@ -598,7 +615,7 @@ def build_style(style):
             build_style(left_base)
 
         cache = left_base.cache
-        
+
     else:
         cache = [ None ] * property_numbers
 
@@ -611,20 +628,20 @@ def build_style(style):
             build_style(ss)
 
         updates.extend(ss.updates)
-        
+
         for i in style.name[1:-1]:
             if (ss.indexed is None) or (i not in ss.indexed):
                 break
 
             ss = ss.indexed[i]
-            
+
             if ss.cache is None:
                 build_style(ss)
 
             updates.extend(ss.updates)
-        
+
     style.updates = my_updates = [ ]
-        
+
     for p in style.properties:
         my_updates.extend(expand_properties(p))
 
@@ -636,20 +653,30 @@ def build_style(style):
             cache[propn] = val
 
     style.cache = cache
-    
-                
+
+
 # This builds all pending styles, recursing to ensure that they are built
 # in the right order.
-def build_styles():
+def build_styles(early=False):
+    """
+    Builds all pending styles.
+
+    `early`
+        If true, builds the pending styles, but leaves the pending queue
+        around, so the styles will be rebuilt later. If false, stops
+        using the pending queue - style changes will be processed
+        immediately.
+    """
 
     global styles_pending
     global styles_built
-    
+
     for s in styles_pending:
         build_style(s)
 
-    styles_pending = None
-    styles_built = True
+    if not early:
+        styles_pending = None
+        styles_built = True
 
 
 def rebuild():
@@ -661,19 +688,19 @@ def rebuild():
 
     if styles_pending is None:
         styles_pending = [ ]
-    
+
     styles_pending += [ j for i in style_parts.values() for j in i.values() ]
     styles_built = False
-    
+
     for i in styles_pending:
         i.cache = None
-        
+
     build_styles()
 
-    
+
 def backup():
     rv = { }
-    
+
     for first, parts in style_parts.iteritems():
         for rest, v in parts.iteritems():
             rv[first, rest] = (v.parent, v.properties[:])
@@ -684,7 +711,7 @@ def backup():
 def restore(o):
     global styles_built
     global styles_pending
-    
+
     styles_pending = [ ]
     styles_built = False
 
@@ -705,7 +732,7 @@ def style_metaclass(name, bases, attrs):
 
         def getter_a(self, k=k):
             return self.getattr(k)
-                        
+
         attrs[k] = property(getter_a, setter_a, deleter_a)
 
 
@@ -746,18 +773,18 @@ class Style(object):
 
         for i in self.__slots__:
             rv[i] = getattr(self, i)
-        
+
         del rv["cache"]
         del rv["offset"]
         del rv["updates"]
-        
+
         return rv
 
     def __setstate__(self, state):
 
         state.pop("heavy", None)
         state.pop("help", None)
-        
+
         for k, v in state.iteritems():
             setattr(self, k, v)
 
@@ -777,10 +804,10 @@ class Style(object):
 
                 if parent is None:
                     raise Exception("The parent of a style must be a named style.")
-            
+
         self.parent = parent
-        
-        
+
+
     def __init__(self, parent, properties=None, heavy=True, name=None, help=None): #@ReservedAssignment
 
         self.prefix = 'insensitive_'
@@ -793,7 +820,7 @@ class Style(object):
 
         self.parent = None
         self.set_parent(parent)
-            
+
         self.indexed = None
         self.cache = None
         self.updates = None
@@ -801,7 +828,7 @@ class Style(object):
 
         if help is not None:
             style_help[self] = help
-        
+
         if properties:
             self.properties.append(properties)
 
@@ -814,10 +841,10 @@ class Style(object):
     def set_prefix(self, prefix):
         self.prefix = prefix
         self.offset = prefix_offset[prefix]
-            
+
     def setattr(self, name, value): #@ReservedAssignment
         self.properties.append({ name : value })
- 
+
     def delattr(self, name): #@ReservedAssignment
 
         for p in self.properties:
@@ -826,13 +853,13 @@ class Style(object):
 
     def getattr(self, name): #@ReservedAssignment
         return self.cache[expansions[name][0][1]]
-                    
+
     def clear(self):
         if styles_built:
             raise Exception("Cannot clear a style after styles have been built.")
         else:
             self.properties = [ ]
-            
+
     def take(self, other):
 
         self.properties = other.properties[:]
@@ -853,20 +880,20 @@ class Style(object):
 
     def __getitem__(self, index):
 
-        if self.indexed is None:            
+        if self.indexed is None:
             self.indexed = { }
 
         if index in self.indexed:
             return self.indexed[index]
 
         name = self.name + (index,)
-        
+
         s = Style(self.parent + (index,), name=name, heavy=not styles_built)
 
         if not styles_built:
             self.indexed[index] = s
             style_parts[name[0]][name[1:]] = s
-            
+
         return s
 
     # This is here to accelerate Displayable.get_placement.
@@ -882,7 +909,7 @@ class Style(object):
             c[o + prop_yoffset],
             c[o + prop_subpixel],
             )
-    
+
 
 def write_text(filename):
 
@@ -893,7 +920,7 @@ def write_text(filename):
 
         return rv
 
-    
+
     f = file(filename, "w")
 
     styles = style_map.items()
@@ -913,17 +940,17 @@ def write_text(filename):
 
         if not sty.cache:
             continue
-            
+
         inherited = [ True ] * property_numbers
-            
+
         for p in sty.properties:
             for _prio, propn, _newval in expand_properties(p):
                 inherited[propn] = False
-            
+
         props = [ (prefix + prop, sty.cache[prefixn + propn], inherited[prefixn + propn])
                   for prefix, prefixn in prefix_offset.iteritems()
                   for prop, propn in property_number.iteritems() ]
-        
+
         props.sort()
 
         for prop, value, inherit in props:
@@ -932,27 +959,27 @@ def write_text(filename):
                 inherit = "(inherited)"
             else:
                 inherit = ""
-            
+
             print >>f, "   ", prop, "=", repr(value), inherit
 
         print >>f
 
     f.close()
-        
-    
+
+
 def style_hierarchy():
     rv = [ ]
     children = { } # Map from parent to list of children.
-    
+
     for v in style_map.values():
 
         if v.parent is not None:
-            parent = style_map        
+            parent = style_map
             for i in v.parent:
                 parent = parent[i]
         else:
             parent = None
-                
+
         children.setdefault(parent, [ ]).append(v)
 
     def recurse(p, depth):
@@ -962,6 +989,6 @@ def style_hierarchy():
             recurse(s, depth + 1)
 
     recurse(None, 0)
-            
+
     return rv
-            
+
