@@ -1,4 +1,4 @@
-# Copyright 2004-2012 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2013 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -51,10 +51,10 @@ class Action(renpy.object.Object):
 
     def predict(self):
         return
-    
+
     def __call__(self):
-        raise NotImplemented()
-    
+        raise Exception("Not implemented")
+
 class BarValue(renpy.object.Object):
     """
     This can be passed to the value method of bar and hotbar.
@@ -65,14 +65,14 @@ class BarValue(renpy.object.Object):
 
     def periodic(self, st):
         return
-    
+
     def get_adjustment(self):
-        raise NotImplemented()
+        raise Exception("Not implemented")
 
     def get_style(self):
         return "bar", "vbar"
 
-    
+
 ##############################################################################
 # Things we can add to. These have two methods: add is called with the
 # widget we're adding. close is called when the thing is ready to be
@@ -97,14 +97,14 @@ class Layer(Addable):
 
         if d and d != self.name:
             raise Exception("ui.close closed layer %s, not the expected %r." % (self.name, d))
-        
+
     def get_layer(self):
         return self.name
 
     def __repr__(self):
         return "<Layer: %r>" % self.name
 
-        
+
 class Many(Addable):
     """
     A widget that takes many children.
@@ -114,7 +114,7 @@ class Many(Addable):
         self.displayable = displayable
         self.imagemap = imagemap
         self.style_group = style_group
-        
+
     def add(self, d, key):
         self.displayable.add(d)
 
@@ -124,27 +124,27 @@ class Many(Addable):
         if self.imagemap:
             imagemap = imagemap_stack.pop()
             imagemap.cache.finish()
-            
-            
+
+
         if d and d != self.displayable:
             raise Exception("ui.close closed %r, not the expected %r." % (self.displayable, d))
 
     def __repr__(self):
         return "<Many: %r>" % self.displayable
-        
+
 class One(Addable):
     """
     A widget that expects exactly one child.
     """
-    
+
     def __init__(self, displayable, style_group):
         self.displayable = displayable
         self.style_group = style_group
-        
+
     def add(self, d, key):
         self.displayable.add(d)
         stack.pop()
-        
+
     def close(self, d):
         raise Exception("Widget %r expects a child." % self.displayable)
 
@@ -161,7 +161,7 @@ class Detached(Addable):
 
     def close(self, d):
         raise Exception("Detached expects to be given a child.")
-    
+
 class ChildOrFixed(Addable):
     """
     If one widget is added, then it is added directly to our
@@ -172,7 +172,7 @@ class ChildOrFixed(Addable):
     def __init__(self, style_group):
         self.queue = [ ]
         self.style_group = style_group
-        
+
     def add(self, d, key):
         self.queue.append(d)
 
@@ -188,7 +188,7 @@ class ChildOrFixed(Addable):
                 add(i)
 
             close()
-                
+
         if d is not None:
             raise Exception("Did not expect to close %r." % d)
 
@@ -203,7 +203,7 @@ add_tag = None
 
 # A stack of Imagemap objects.
 imagemap_stack = [ ]
-          
+
 
 # Called at the end of the init phase, and from the screen
 # prediction code.
@@ -215,29 +215,29 @@ def reset():
     stack = [ Layer('transient') ]
     at_stack = [ ]
     imagemap_stack = [ ]
-    
+
 renpy.game.post_init.append(reset)
-    
+
 def interact(type='misc', roll_forward=None, **kwargs): #@ReservedAssignment
     # Docs in wiki.
 
     if stack is None:
         raise Exception("Interaction not allowed during init phase.")
-    
+
     if renpy.config.skipping == "fast":
         renpy.config.skipping = None
 
     if len(stack) != 1:
         raise Exception("ui.interact called with non-empty widget/layer stack. Did you forget a ui.close() somewhere?")
-    
+
     if at_stack:
         raise Exception("ui.interact called with non-empty at stack.")
-    
+
     renpy.game.context().info._current_interact_type = type
     rv = renpy.game.interface.interact(roll_forward=roll_forward, **kwargs)
     renpy.game.context().mark_seen()
     renpy.game.context().info._last_interact_type = type
-    
+
     if renpy.exports.in_fixed_rollback() and roll_forward is not None:
         return roll_forward
     else:
@@ -256,7 +256,7 @@ def child_or_fixed():
     """
 
     stack.append(ChildOrFixed(stack[-1].style_group))
-    
+
 def remove(d):
     layer = stack[-1].get_layer()
     renpy.game.context(-1).scene_lists.remove(layer, d)
@@ -264,16 +264,16 @@ def remove(d):
 def remove_above(d):
     layer = stack[-1].get_layer()
     renpy.game.context(-1).scene_lists.remove_above(layer, d)
-    
+
 def at(transform):
     """
     :doc: ui
-    
+
     Specifieds a transform that is applied to the next displayable to
     be created. This is largely obsolete, as all UI functions now take
     an `at` argument.
     """
-    
+
     at_stack.append(transform)
 
 def clear():
@@ -287,9 +287,9 @@ def detached():
     Do not add the next displayable to any later or container. Use this if
     you want to assign the result of a ui function to a variable.
     """
-    
+
     stack.append(Detached())
-    
+
 def layer(name):
     """
     :doc: ui
@@ -299,7 +299,7 @@ def layer(name):
     """
 
     stack.append(Layer(name))
-    
+
 def close(d=None):
     """
     :doc: ui
@@ -314,7 +314,7 @@ def close(d=None):
 
     if not stack:
         raise Exception("ui.close() called when no layer or widget is open.")
-    
+
 def reopen(w, clear):
 
     stack.append(Many(w))
@@ -332,19 +332,19 @@ def context_exit(w):
     close(w)
 
 NoStyleGroupGiven = object()
-    
+
 def style_group_style(s, style_group):
     """
     Given a style name s, combine it with the style_group to create a new
     style. If the style doesn't exist, create a new lightweight style.
     """
-    
+
     if style_group is NoStyleGroupGiven:
         style_group = stack[-1].style_group
 
     if style_group is None:
         return s
-    
+
     new_style = style_group + "_" + s
 
     if new_style[0] == "_":
@@ -352,18 +352,18 @@ def style_group_style(s, style_group):
 
     if new_style not in renpy.style.style_map:
         renpy.style.style_map[new_style] = renpy.style.Style(s, heavy=False, name=new_style)
-        
+
     return new_style
-        
+
 # The screen we're using as we add widgets. None if there isn't a
 # screen.
 screen = None
-    
+
 class Wrapper(renpy.object.Object):
 
     def __reduce__(self):
         return self.name
-    
+
     def __init__(self, function, one=False, many=False, imagemap=False, replaces=False, style=None, **kwargs):
 
         # The name assigned to this wrapper. This is used to serialize us correctly.
@@ -380,20 +380,20 @@ class Wrapper(renpy.object.Object):
         # Should the function be given the replaces parameter,
         # specifiying the displayable it replaced?
         self.replaces = replaces
-        
+
         # Default keyword arguments to the function.
         self.kwargs = kwargs
 
         # Default style (suffix).
         self.style = style
-        
+
     def __call__(self, *args, **kwargs):
 
         global add_tag
 
         if not stack:
             raise Exception("Can't add displayable during init phase.")
-        
+
         # Pull out the special kwargs, id, at, and style_group.
 
         id = kwargs.pop("id", None) #@ReservedAssignment
@@ -407,7 +407,7 @@ class Wrapper(renpy.object.Object):
         # Figure out our style_group.
         if style_group is NoStyleGroupGiven:
             style_group = stack[-1].style_group
-        
+
         # Figure out the keyword arguments, based on the parameters.
         if self.kwargs:
             keyword = self.kwargs.copy()
@@ -420,30 +420,30 @@ class Wrapper(renpy.object.Object):
 
         # Should we add?
         do_add = True
-        
+
         if screen:
             if id in screen.widget_properties:
                 keyword.update(screen.widget_properties[id])
 
             if id in screen.hidden_widgets:
                 do_add = False
-                
+
         grab = False
-                
+
         if old_transfers:
             if self.replaces:
                 w = screen.old_widgets.get(id, None)
 
                 if w is not None:
                     keyword["replaces"] = w
-                    
-                    if renpy.display.focus.grab is w:
+
+                    if (renpy.display.focus.grab is w) and (not screen.hiding):
                         grab = True
-                        
+
 
         if self.style and "style" not in keyword:
             keyword["style"] = style_group_style(self.style, style_group)
-            
+
         try:
             w = self.function(*args, **keyword)
         except TypeError, e:
@@ -456,14 +456,14 @@ class Wrapper(renpy.object.Object):
             raise
 
         # Wrap the displayable based on the at_list and at_stack.
-        atw = w 
+        atw = w
 
         if grab:
             renpy.display.focus.new_grab = w
 
         while at_stack:
             at_list.append(at_stack.pop())
-        
+
         for atf in at_list:
             if isinstance(atf, renpy.display.motion.Transform):
                 atw = atf(child=atw)
@@ -473,7 +473,7 @@ class Wrapper(renpy.object.Object):
         # Add to the displayable at the bottom of the stack.
         if do_add:
             stack[-1].add(atw, add_tag)
-            
+
         # Update the stack, as necessary.
         if self.one:
             stack.append(One(w, style_group))
@@ -491,11 +491,11 @@ class Wrapper(renpy.object.Object):
                 if old_transfers:
 
                     oldt = screen.old_transforms.get(id, None)
-                        
+
                     if oldt is not None:
                         atw.take_state(oldt)
                         atw.take_execution_state(oldt)
-                    
+
         # Clear out the add_tag.
         add_tag = None
 
@@ -525,17 +525,17 @@ def is_sensitive(clicked):
     else:
         return True
 
-    
+
 
 ##############################################################################
 # Widget functions.
-        
+
 def _add(d, **kwargs):
     d = renpy.easy.displayable(d)
     d = d.parameterize('displayable', [ ])
-    
+
     rv = d
-    
+
     if kwargs:
         rv = renpy.display.motion.Transform(child=d, **kwargs)
 
@@ -547,7 +547,7 @@ def _image(im, **properties):
     d = renpy.display.im.image(im, loose=True, **properties)
     d = d.parameterize('displayable', [ ])
     return d
-    
+
 image = Wrapper(_image)
 
 null = Wrapper(renpy.display.layout.Null)
@@ -574,7 +574,7 @@ def _key(key, action=None):
 
     if action is None:
         raise Exception("Action is required in ui.key.")
-    
+
     return renpy.display.behavior.Keymap(**{ key : action})
 
 key = Wrapper(_key)
@@ -585,14 +585,14 @@ class ChoiceActionBase(Action):
     and value. The class will automatically determine the rollback state
     and supply correct "sensitive" and "selected" information to the
     widget.
-    If a location is supplied, it will check whether the choice was 
+    If a location is supplied, it will check whether the choice was
     previously visited and mark it so if it is chosen.
     """
     def __init__(self, label, value, location=None, block_all=None):
         self.label = label
         self.value = value
         self.location = location
-        
+
         if block_all is None:
             self.block_all = renpy.config.fix_rollback_without_choice
         else:
@@ -601,7 +601,7 @@ class ChoiceActionBase(Action):
         self.chosen = None
 
         if self.location:
-            self.chosen = renpy.game.persistent._chosen
+            self.chosen = renpy.game.persistent._chosen  # @UndefinedVariable
 
             if self.chosen is None:
                 self.chosen = renpy.game.persistent._chosen = { }
@@ -612,14 +612,14 @@ class ChoiceActionBase(Action):
 
     def get_selected(self):
         roll_forward = renpy.exports.roll_forward_info()
-        return renpy.exports.in_fixed_rollback() and roll_forward == self.value 
+        return renpy.exports.in_fixed_rollback() and roll_forward == self.value
 
     def get_chosen(self):
         if self.chosen is None:
             return False
-        
+
         return (self.location, self.label) in self.chosen
-    
+
 class ChoiceReturn(ChoiceActionBase):
     """
     :doc: blockrollback
@@ -627,26 +627,26 @@ class ChoiceReturn(ChoiceActionBase):
     A menu choice action that returns `value`, while managing the button
     state in a manner consistent with fixed rollback. (See block_all for
     a description of the behavior.)
-    
-    
+
+
     `label`
         The label text of the button. For imagebuttons and hotspots this
         can be anything. This label is used as a unique identifier of
         the options within the current screen. Together with `location`
         it is used to store whether this option has been chosen.
-        
+
     `value`
         The value this is returned when the choice is chosen.
 
     `location`
         A unique location identifier for the current choices screen.
-        
+
     `block_all`
         If false, the button is given the selected role if it was
-        the chosen choice, and insensitive if it was not selected.        
+        the chosen choice, and insensitive if it was not selected.
 
-        If true, the button is always insensitive during fixed 
-        rollback. 
+        If true, the button is always insensitive during fixed
+        rollback.
 
         If None, the value is taken from the :var:`config.fix_rollback_without_choice`
         variable.
@@ -656,13 +656,13 @@ class ChoiceReturn(ChoiceActionBase):
         be changed by calling :func:`ui.saybehavior` before the call
         to :func:`ui.interact`.
     """
-    
 
-    
+
+
     def __call__(self):
         if self.chosen is not None:
             self.chosen[(self.location, self.label)] = True
-        
+
         return self.value
 
 class ChoiceJump(ChoiceActionBase):
@@ -672,26 +672,26 @@ class ChoiceJump(ChoiceActionBase):
     A menu choice action that returns `value`, while managing the button
     state in a manner consistent with fixed rollback. (See block_all for
     a description of the behavior.)
-    
-    
+
+
     `label`
         The label text of the button. For imagebuttons and hotspots this
         can be anything. This label is used as a unique identifier of
         the options within the current screen. Together with `location`
         it is used to store whether this option has been chosen.
-        
+
     `value`
         The location to jump to.
-        
+
     `location`
         A unique location identifier for the current choices screen.
-        
+
     `block_all`
         If false, the button is given the selected role if it was
-        the chosen choice, and insensitive if it was not selected.        
+        the chosen choice, and insensitive if it was not selected.
 
-        If true, the button is always insensitive during fixed 
-        rollback. 
+        If true, the button is always insensitive during fixed
+        rollback.
 
         If None, the value is taken from the :var:`config.fix_rollback_without_choice`
         variable.
@@ -701,8 +701,8 @@ class ChoiceJump(ChoiceActionBase):
         be changed by calling :func:`ui.saybehavior` before the call
         to :func:`ui.interact`.
     """
-    
-    
+
+
     def get_selected(self):
         roll_forward = renpy.exports.roll_forward_info()
 
@@ -710,7 +710,7 @@ class ChoiceJump(ChoiceActionBase):
         if isinstance(roll_forward, renpy.game.JumpException):
             roll_forward = roll_forward.args[0]
 
-        return renpy.exports.in_fixed_rollback() and roll_forward == self.value 
+        return renpy.exports.in_fixed_rollback() and roll_forward == self.value
 
     def __call__(self):
         if self.chosen is not None:
@@ -742,7 +742,7 @@ def menu(menuitems,
 
             text = choice_style
             button = choice_button_style
-                        
+
             clicked = ChoiceReturn(label, val, location)
 
             if clicked.get_chosen():
@@ -756,16 +756,16 @@ def menu(menuitems,
 
             button = button[label]
             text = text[label]
-            
+
             renpy.ui.textbutton(label,
                                 style=button,
                                 text_style=text,
                                 clicked=clicked,
                                 focus=focus,
                                 default=default)
-            
+
     close()
-    
+
 input = Wrapper(renpy.display.behavior.Input, exclude='{}', style="input", replaces=True) #@ReservedAssignment
 
 def imagemap_compat(ground,
@@ -790,11 +790,11 @@ def imagemap_compat(ground,
 
         if result is None:
             continue
-            
+
         action = ChoiceReturn(result, result)
-        
+
         selected_img = renpy.display.layout.LiveCrop((x0, y0, x1 - x0, y1 - y0), selected)
-        
+
         imagebutton(renpy.display.layout.LiveCrop((x0, y0, x1 - x0, y1 - y0), unselected),
                     selected_img,
                     selected_idle_image = selected_img,
@@ -813,13 +813,13 @@ def imagemap_compat(ground,
 button = Wrapper(renpy.display.behavior.Button, style='button', one=True)
 
 def _imagebutton(idle_image = None,
-                 hover_image = None,                 
+                 hover_image = None,
                  insensitive_image = None,
                  activate_image = None,
                  selected_idle_image = None,
                  selected_hover_image = None,
                  selected_insensitive_image = None,
-                 selected_activate_image = None,    
+                 selected_activate_image = None,
                  idle=None,
                  hover=None,
                  insensitive=None,
@@ -841,14 +841,14 @@ def _imagebutton(idle_image = None,
             return renpy.config.imagemap_auto_function(auto, name)
 
         return None
-    
+
     idle = choice(idle, idle_image, "idle")
     hover = choice(hover, hover_image, "hover")
     insensitive = choice(insensitive, insensitive_image, "insensitive")
     selected_idle = choice(selected_idle, selected_idle_image, "selected_idle")
     selected_hover = choice(selected_hover, selected_hover_image, "selected_hover")
     selected_insensitive = choice(selected_insensitive, selected_insensitive_image, "selected_insensitive")
-    
+
     return renpy.display.behavior.ImageButton(
             idle,
             hover,
@@ -857,9 +857,9 @@ def _imagebutton(idle_image = None,
             selected_idle_image = selected_idle,
             selected_hover_image = selected_hover,
             selected_insensitive_image = selected_insensitive,
-            selected_activate_image = selected_activate_image,    
+            selected_activate_image = selected_activate_image,
             **properties)
-    
+
 imagebutton = Wrapper(_imagebutton, style="image_button")
 
 def get_text_style(style, default):
@@ -893,12 +893,19 @@ def textbutton(label, clicked=None, style=None, text_style=None, substitute=True
         else:
             button_kwargs[k] = v
 
+    # Deal with potentially bad keyword arguments. (We'd get these if the user
+    # writes text_align instead of text_text_align.)
+    if "align" in text_kwargs:
+        if isinstance(text_kwargs["align"], float):
+            text_kwargs.pop("align")
+    text_kwargs.pop("y_fudge", None)
+
     if style is None:
         style = style_group_style('button', NoStyleGroupGiven)
-    
+
     if text_style is None:
         text_style = get_text_style(style, style_group_style('button_text', NoStyleGroupGiven))
-        
+
     button(style=style, clicked=clicked, **button_kwargs)
     text(label, style=text_style, substitute=substitute, scope=scope, **text_kwargs)
 
@@ -915,10 +922,10 @@ def label(label, style=None, text_style=None, substitute=True, scope=None, **kwa
 
     if style is None:
         style = style_group_style('label', NoStyleGroupGiven)
-    
+
     if text_style is None:
         text_style = get_text_style(style, style_group_style('label_text', NoStyleGroupGiven))
-        
+
     window(style=style, **label_kwargs)
     text(label, style=text_style, substitute=substitute, scope=scope, **text_kwargs)
 
@@ -946,7 +953,7 @@ def _bar(*args, **properties):
 
     if "range" in properties:
         range = properties.pop("range") #@ReservedAssignment
-         
+
     if "value" in properties:
         value = properties.pop("value")
 
@@ -959,9 +966,9 @@ def _bar(*args, **properties):
 
             if isinstance(style, basestring):
                 style = style_group_style(style, NoStyleGroupGiven)
-                
+
             properties["style"] = style
-        
+
     return renpy.display.behavior.Bar(range, value, width, height, **properties)
 
 bar = Wrapper(_bar, vertical=False, replaces=True)
@@ -979,7 +986,7 @@ def _autobar_interpolate(range, start, end, time, st, at, **properties): #@Reser
     else:
         t = st / time
         redraw = 0
-        
+
     value = start + t * (end - start)
     return renpy.display.behavior.Bar(range, value, None, None, **properties), redraw
 
@@ -1008,43 +1015,43 @@ def viewport(scrollbars=None, **properties):
 
     if scrollbars == "vertical":
         side("c r", **side_properties)
-        
+
         rv = _viewport(**viewport_properties)
         addable = stack.pop()
-        
+
         vscrollbar(adjustment=rv.yadjustment)
         close()
 
         stack.append(addable)
-        
+
         return rv
-        
+
     elif scrollbars == "horizontal":
         side("c b", **side_properties)
-        
+
         rv = _viewport(**viewport_properties)
         addable = stack.pop()
-        
+
         scrollbar(adjustment=rv.xadjustment)
         close()
 
         stack.append(addable)
-        
+
         return rv
 
     else:
-    
+
         side("c r b", **side_properties)
-        
+
         rv = _viewport(**viewport_properties)
         addable = stack.pop()
-        
+
         vscrollbar(adjustment=rv.yadjustment)
         scrollbar(adjustment=rv.xadjustment)
         close()
 
         stack.append(addable)
-        
+
         return rv
 
 conditional = Wrapper(renpy.display.behavior.Conditional, one=True)
@@ -1063,7 +1070,7 @@ class Imagemap(object):
     """
 
     alpha = True
-    
+
     def __init__(self, insensitive, idle, selected_idle, hover, selected_hover, selected_insensitive, alpha, cache):
         self.insensitive = renpy.easy.displayable(insensitive)
         self.idle = renpy.easy.displayable(idle)
@@ -1075,7 +1082,7 @@ class Imagemap(object):
         self.alpha = alpha
 
         self.cache = renpy.display.imagemap.ImageMapCache(cache)
-        
+
 def _imagemap(ground=None, hover=None, insensitive=None, idle=None, selected_hover=None, selected_idle=None, selected_insensitive=None, auto=None, alpha=True, cache=True, style='imagemap', **properties):
 
     def pick(variable, name, other):
@@ -1113,14 +1120,14 @@ def _imagemap(ground=None, hover=None, insensitive=None, idle=None, selected_hov
             cache))
 
     properties.setdefault('fit_first', True)
-    
+
     rv = renpy.display.layout.MultiBox(layout='fixed', **properties)
 
     if ground:
         rv.add(renpy.easy.displayable(ground))
-        
+
     return rv
-    
+
 imagemap = Wrapper(_imagemap, imagemap=True, style='imagemap')
 
 def _hotspot(spot, style='imagemap_button', **properties):
@@ -1138,14 +1145,14 @@ def _hotspot(spot, style='imagemap_button', **properties):
     selected_hover = imagemap.selected_hover
     insensitive = imagemap.insensitive
     selected_insensitive = imagemap.selected_insensitive
-    
+
     idle = imagemap.cache.crop(idle, spot)
     hover = imagemap.cache.crop(hover, spot)
     selected_idle = imagemap.cache.crop(selected_idle, spot)
-    selected_hover = imagemap.cache.crop(selected_hover, spot) 
+    selected_hover = imagemap.cache.crop(selected_hover, spot)
     insensitive = imagemap.cache.crop(insensitive, spot)
     selected_insensitive = imagemap.cache.crop(selected_insensitive, spot)
-            
+
     properties.setdefault("xpos", x)
     properties.setdefault("xanchor", 0)
     properties.setdefault("ypos", y)
@@ -1161,7 +1168,7 @@ def _hotspot(spot, style='imagemap_button', **properties):
         focus_mask = None
 
     properties.setdefault("focus_mask", focus_mask)
-        
+
     return renpy.display.behavior.Button(
         None,
         idle_background=idle,
@@ -1207,11 +1214,11 @@ def _hotbar(spot, adjustment=None, range=None, value=None, **properties): #@Rese
 
         fore_bar, aft_bar = aft_bar, fore_bar
         hover_fore_bar, hover_aft_bar = hover_aft_bar, hover_fore_bar
-        
+
     return renpy.display.behavior.Bar(
             adjustment=adjustment,
             range=range,
-            value=value,            
+            value=value,
             fore_bar=fore_bar,
             aft_bar=aft_bar,
             hover_fore_bar=hover_fore_bar,
@@ -1226,8 +1233,8 @@ def _hotbar(spot, adjustment=None, range=None, value=None, **properties): #@Rese
             ymaximum=h,
             **properties)
 
-hotbar = Wrapper(_hotbar, style="hotbar")
-    
+hotbar = Wrapper(_hotbar, style="hotbar", replaces=True)
+
 
 ##############################################################################
 # Curried functions, for use in clicked, hovered, and unhovered.
@@ -1245,7 +1252,7 @@ def _jumps(label, transition=None):
 
     if transition is not None:
         renpy.exports.transition(transition)
-    
+
     raise renpy.exports.jump(label)
 
 jumps = renpy.curry.curry(_jumps)
